@@ -3,7 +3,7 @@
  *
  * Copyright (c) 2007 Ryusuke SEKIYAMA. All rights reserved.
  * Copyright (c) 2009 Keynetics Inc. All rights reserved.
- * Copyright (c) 2012 Bryan Davis All rights reserved.
+ * Copyright (c) 2015 Bryan Davis and contributors. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -25,16 +25,16 @@
  *
  * @package     php_yaml
  * @author      Ryusuke SEKIYAMA <rsky0711@gmail.com>
- * @author      Bryan Davis <bpd@keynetics.com>
+ * @author      Bryan Davis <bd808@bd808.com>
  * @copyright   2007 Ryusuke SEKIYAMA
  * @copyright   2009 Keynetics Inc
+ * @copyright   2015 Bryan Davis and contributors
  * @license     http://www.opensource.org/licenses/mit-license.php  MIT License
- * @version     SVN: $Id$
  */
 
 
 #include "php_yaml.h"
-#include "zval_refcount.h"		/* for PHP < 5.3 */
+#include "zval_refcount.h"  /* for PHP < 5.3 */
 #include "php_yaml_int.h"
 
 /* {{{ local macros
@@ -91,7 +91,7 @@ static int apply_filter(
 
 static char *convert_to_char(zval *zv TSRMLS_DC);
 
-static int eval_timestamp(zval **zpp, char *ts, int ts_len TSRMLS_DC);
+static int eval_timestamp(zval **zpp, const char *ts, size_t ts_len TSRMLS_DC);
 
 /* }}} */
 
@@ -394,7 +394,7 @@ static zval *handle_document(parser_state_t *state TSRMLS_DC)
 static zval *handle_mapping(parser_state_t *state TSRMLS_DC)
 {
 	zval *retval = { 0 };
-	yaml_event_t src_event = { 0 }, key_event = { 0 };
+	yaml_event_t src_event = { YAML_NO_EVENT }, key_event = { YAML_NO_EVENT };
 	zval *key = { 0 };
 	char *key_str;
 	zval *value = { 0 };
@@ -495,7 +495,7 @@ static zval *handle_mapping(parser_state_t *state TSRMLS_DC)
  */
 static zval *handle_sequence (parser_state_t *state TSRMLS_DC) {
 	zval *retval = { 0 };
-	yaml_event_t src_event = { 0 };
+	yaml_event_t src_event = { YAML_NO_EVENT };
 	zval *value = { 0 };
 
 	/* save copy of sequence start event */
@@ -687,6 +687,17 @@ zval *eval_scalar(yaml_event_t event,
 	MAKE_STD_ZVAL(retval);
 	ZVAL_NULL(retval);
 
+	/* check for non-specific tag (treat as a string) */
+	if (SCALAR_TAG_IS(event, YAML_NONSPECIFIC_TAG) ||
+			event.data.scalar.quoted_implicit) {
+#ifdef IS_UNICODE
+		ZVAL_U_STRINGL(UG(utf8_conv), retval, value, length, ZSTR_DUPLICATE);
+#else
+		ZVAL_STRINGL(retval, value, length, 1);
+#endif
+		return retval;
+	}
+
 	/* check for null */
 	if (scalar_is_null(value, length, &event)) {
 		return retval;
@@ -743,51 +754,38 @@ zval *eval_scalar(yaml_event_t event,
 	}
 
 	/* check for timestamp */
-	if (event.data.scalar.plain_implicit || event.data.scalar.quoted_implicit) {
-		if (scalar_is_timestamp(value, length)) {
-			if (FAILURE == eval_timestamp(
-					&retval, value, (int) length TSRMLS_CC)) {
-				zval_ptr_dtor(&retval);
-				return NULL;
-			}
-			return retval;
-		}
-
-	} else if (SCALAR_TAG_IS(event, YAML_TIMESTAMP_TAG)) {
+	if (IS_NOT_IMPLICIT_AND_TAG_IS(event, YAML_TIMESTAMP_TAG) ||
+			 scalar_is_timestamp(value, length)) {
 		if (FAILURE == eval_timestamp(
 				&retval, value, (int) length TSRMLS_CC)) {
-			zval_ptr_dtor(&retval);
-			return NULL;
+			ZVAL_NULL(retval);
 		}
 		return retval;
 	}
 
 	/* check for binary */
-	if (IS_NOT_IMPLICIT_AND_TAG_IS(event, YAML_BINARY_TAG)) {
-		if (YAML_G(decode_binary)) {
-			unsigned char *data = { 0 };
-			int data_len = 0;
+	if (YAML_G(decode_binary) &&
+			IS_NOT_IMPLICIT_AND_TAG_IS(event, YAML_BINARY_TAG)) {
+		unsigned char *data = { 0 };
+		int data_len = 0;
 
-			data = php_base64_decode(
-					(const unsigned char *) value, (int) length, &data_len);
-			if (NULL == data) {
-				php_error_docref(NULL TSRMLS_CC, E_WARNING,
-						"Failed to decode binary");
-				ZVAL_NULL(retval);
-
-			} else {
-				ZVAL_STRINGL(retval, (char *) data, data_len, 0);
-			}
+		data = php_base64_decode(
+				(const unsigned char *) value, (int) length, &data_len);
+		if (NULL == data) {
+			php_error_docref(NULL TSRMLS_CC, E_WARNING,
+					"Failed to decode binary");
+			ZVAL_NULL(retval);
 
 		} else {
-			ZVAL_STRINGL(retval, value, length, 1);
+			ZVAL_STRINGL(retval, (char *) data, data_len, 0);
 		}
 
 		return retval;
 	}
 
 	/* check for php object */
-	if (IS_NOT_IMPLICIT_AND_TAG_IS(event, YAML_PHP_TAG)) {
+	if (YAML_G(decode_php) &&
+			IS_NOT_IMPLICIT_AND_TAG_IS(event, YAML_PHP_TAG)) {
 		const unsigned char *p;
 		php_unserialize_data_t var_hash;
 
@@ -796,7 +794,6 @@ zval *eval_scalar(yaml_event_t event,
 
 		if (!php_var_unserialize(
 				&retval, &p, p + (int) length, &var_hash TSRMLS_CC)) {
-			PHP_VAR_UNSERIALIZE_DESTROY(var_hash);
 			php_error_docref(NULL TSRMLS_CC, E_NOTICE,
 					"Failed to unserialize class");
 			/* return the serialized string directly */
@@ -820,13 +817,13 @@ zval *eval_scalar(yaml_event_t event,
 
 
 /* {{{ eval_scalar_with_callbacks()
- * Convert a scalar node to the proper PHP data type using user supplied input 
+ * Convert a scalar node to the proper PHP data type using user supplied input
  * filters if available.
  */
 zval *eval_scalar_with_callbacks(yaml_event_t event,
 		HashTable *callbacks TSRMLS_DC)
 {
-	char *tag = (char *) event.data.scalar.tag;
+	const char *tag = (char *) event.data.scalar.tag;
 	zval **callback = { 0 };
 
 	if (YAML_PLAIN_SCALAR_STYLE == event.data.scalar.style && NULL == tag) {
@@ -1002,14 +999,14 @@ static char *convert_to_char(zval *zv TSRMLS_DC)
  *  - yaml.decode_timestamp=2 for date_create parsing
  */
 static int
-eval_timestamp(zval ** zpp, char *ts, int ts_len TSRMLS_DC)
+eval_timestamp(zval **zpp, const char *ts, size_t ts_len TSRMLS_DC)
 {
 	if (NULL != YAML_G(timestamp_decoder) ||
 			1L == YAML_G(decode_timestamp) ||
 			2L == YAML_G(decode_timestamp)) {
 		zval **argv[] = { NULL };
 		zval *arg, *retval, *func, afunc;
-		char *funcs[] = { "strtotime", "date_create" };
+		const char *funcs[] = { "strtotime", "date_create" };
 
 		INIT_ZVAL(afunc);
 		if (NULL == YAML_G(timestamp_decoder)) {
@@ -1040,14 +1037,14 @@ eval_timestamp(zval ** zpp, char *ts, int ts_len TSRMLS_DC)
 
 			while (src < end && *src != '.') {
 				if (src + 1 < end &&
-						(*(src - 1) >= '0' && *(src - 1) <= '9') && 
-						(*src == 'T' || *src == 't') && 
+						(*(src - 1) >= '0' && *(src - 1) <= '9') &&
+						(*src == 'T' || *src == 't') &&
 						(*(src + 1) >= '0' && *(src + 1) <= '9')) {
 					src++;
 					*dst++ = ' ';
 
 				} else if (*src == ':' && src > ts + 2 && (
-						((*(src - 2) == '+' || *(src - 2) == '-') && 
+						((*(src - 2) == '+' || *(src - 2) == '-') &&
 						 (*(src - 1) >= '0' || *(src - 1) <= '5')) ||
 						((*(src - 3) == '+' || *(src - 3) == '-') &&
 						 (*(src - 2) >= '0' || *(src - 2) <= '5') &&
