@@ -3,7 +3,7 @@
  *
  * Copyright (c) 2007 Ryusuke SEKIYAMA. All rights reserved.
  * Copyright (c) 2009 Keynetics Inc. All rights reserved.
- * Copyright (c) 2012 Bryan Davis All rights reserved.
+ * Copyright (c) 2015 Bryan Davis and contributors. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -25,12 +25,11 @@
  *
  * @package     php_yaml
  * @author      Ryusuke SEKIYAMA <rsky0711@gmail.com>
- * @author      Bryan Davis <bpd@keynetics.com>
+ * @author      Bryan Davis <bd808@bd808.com>
  * @copyright   2007 Ryusuke SEKIYAMA
  * @copyright   2009 Keynetics Inc
- * @copyright   2012 Bryan Davis
+ * @copyright   2015 Bryan Davis and contributors
  * @license     http://www.opensource.org/licenses/mit-license.php  MIT License
- * @version     SVN: $Id$
  */
 
 
@@ -65,9 +64,10 @@ typedef struct y_emit_state_s {
 
 /* {{{ ext/yaml macros
 */
-#define YAML_BINARY_TAG     "tag:yaml.org,2002:binary"
-#define YAML_MERGE_TAG      "tag:yaml.org,2002:merge"
-#define YAML_PHP_TAG        "!php/object"
+#define YAML_BINARY_TAG      "tag:yaml.org,2002:binary"
+#define YAML_MERGE_TAG       "tag:yaml.org,2002:merge"
+#define YAML_PHP_TAG         "!php/object"
+#define YAML_NONSPECIFIC_TAG "!"
 
 #define Y_SCALAR_IS_NOT_NUMERIC 0x00
 #define Y_SCALAR_IS_INT         0x10
@@ -91,16 +91,25 @@ typedef struct y_emit_state_s {
 #endif
 
 #define STR_EQ(a, b)\
-	(0 == strcmp(a, b))
+	(a != NULL && b != NULL && 0 == strcmp(a, b))
 
 #define SCALAR_TAG_IS(event, name) \
-	STR_EQ((const char *)event.data.scalar.tag, name)
+	STR_EQ(name, (const char *)event.data.scalar.tag)
+
+#define IS_NOT_IMPLICIT(event) \
+	(!event.data.scalar.quoted_implicit && !event.data.scalar.plain_implicit)
 
 #define IS_NOT_IMPLICIT_AND_TAG_IS(event, name) \
-	(!event.data.scalar.quoted_implicit && !event.data.scalar.plain_implicit && SCALAR_TAG_IS(event, name))
+	(IS_NOT_IMPLICIT(event) && SCALAR_TAG_IS(event, name))
+
+#define IS_NOT_QUOTED(event) \
+	(YAML_PLAIN_SCALAR_STYLE == event.data.scalar.style || YAML_ANY_SCALAR_STYLE == event.data.scalar.style)
 
 #define IS_NOT_QUOTED_OR_TAG_IS(event, name) \
-	((YAML_PLAIN_SCALAR_STYLE == event.data.scalar.style || YAML_ANY_SCALAR_STYLE == event.data.scalar.style) && (event.data.scalar.plain_implicit || SCALAR_TAG_IS(event, name)))
+	(IS_NOT_QUOTED(event) && (event.data.scalar.plain_implicit || SCALAR_TAG_IS(event, name)))
+
+#define SCALAR_IS_QUOTED(event) \
+	(YAML_SINGLE_QUOTED_SCALAR_STYLE == event.data.scalar.style || YAML_DOUBLE_QUOTED_SCALAR_STYLE == event.data.scalar.style)
 
 /* }}} */
 
@@ -116,7 +125,7 @@ zval *eval_scalar(yaml_event_t event, HashTable *callbacks TSRMLS_DC);
 zval *eval_scalar_with_callbacks(
 		yaml_event_t event, HashTable *callbacks TSRMLS_DC);
 
-char *detect_scalar_type(
+const char *detect_scalar_type(
 		const char *value, size_t length, const yaml_event_t *event);
 
 int scalar_is_null(
